/************************************************************************/
/*      pp.c            print-picture file

        by Rolf Loh
*/
/************************************************************************/

#include <stdio.h>
#include <sgtty.h>
#include "fedef.h"                      /* Include definitions          */


/************************************************************************/
/*                         D A T A      A R E A                         */

char picarea[ 0x7fff ];         /* work fontfile area                   */
char utbuf[2048];
/*                                                                      */ 


#define NCC     8

typedef unsigned char   uchar_t;
#define IXOFF   0010000


#define TIOC    ('T'<<8)
#define TCGETA  (TIOC|1)
#define TCSETA  (TIOC|2)

/*
 * Ioctl control packet
 */
struct termio {
        unsigned short  c_iflag;        /* input modes */
        unsigned short  c_oflag;        /* output modes */
        unsigned short  c_cflag;        /* control modes */
        unsigned short  c_lflag;        /* line discipline modes */
        char            c_line;         /* line discipline */
        uchar_t         c_ccs[NCC];     /* control chars */
};



char c;
int bytcnt;
int pixwidth,i,x;
char str[64];
int inv_flag;

struct sgttyb ttysave;
struct pictblk pic_hdr;



/************************************************************************/



main(argc, argv)
int argc;
char *argv[];
{
        struct termio term;
        int fp1,fp2;

        if( argc < 3) {
                error("Usage: pp fromfile /dev/lp [invert]",NULL);
                exit(0);
        }

        /* kolla om bilden skall inverteras */
        inv_flag=0;
        if( argc == 4 ) inv_flag = 1;


        /* avkoda kommandoraden         */
        if(( fp1 = open(argv[1],0)) == -1) {
                error("pp: can't open %s\n", argv[1]);
                exit(0);
        }
        if(( fp2 = creat(argv[2], 0644)) ==-1) {
                error("pp: can't create %s\n", argv[2]);
                exit(0);
        }
 
        /* set tty-mode */

        ioctl(fp2,TIOCGETP,&ttysave);
        ttysave.sg_flags |= RAW;        /* s{tt raw mode        */
        ttysave.sg_flags &= ~ECHO;      /* ta bort echo         */
        ioctl(fp2,TIOCSETP,&ttysave);

        /* disable automatic XON/XOFF */
        /* should be done after setting the line to RAW mode */
        ioctl(fp2,TCGETA,&term);
        term.c_iflag &= ~IXOFF;
        ioctl(fp2,TCSETA,&term);
    

        prcr(fp2);
        prlf(fp2);
        cstr();
        sprintf( &str[0], "     Picturefile:%s\n\n",&*argv[1]);
        write(fp2, &str[0], 64 ); 

        fontprint(fp1, fp2);    /* skriv ut bilden              */

        ttysave.sg_flags &= ~RAW;       /* }terst{ll porten     */
        ttysave.sg_flags |= ECHO;
        ioctl(fp2,TIOCSETP,&ttysave);

        exit(0);
}

/****************************************************************/
error(s1,s2)    /* skriver ut felutskrift.s1=str{ng,s2s{tts in i */
char *s1,*s2;   /* s1 med %s funktonen i printf                 */
{
        printf(s1, s2);
        printf("\n");
        exit(1);
}


fontprint(fp1,fp2)
/* Denna rutin l{ser f|rst header(50 bytes).Sedan l{ses det antal bytes */
/* som kr{vs f|r att skriva med 7 n}lar |ver bildens bredd.D{refter     */
/* konverteras dessa bytes f|r att skrivas ut.                          */
int fp1,fp2;
{
        /* Read header          */
        bytcnt = read( fp1,(char*)&pic_hdr,sizeof(pic_hdr));


        pixwidth = 8 * pic_hdr.bwidth  ;

        prcr(fp2);
        prlf(fp2);
        prnull(fp2);

        bytcnt = 1;
        while( bytcnt != 0 ) {
                /* Read data            */
                bytcnt = read( fp1,&picarea[0],7*pic_hdr.bwidth); /*Read data*/
                prconv( &picarea[0], pic_hdr.bwidth,pixwidth);
                prout( fp2,&utbuf[0], pixwidth );
                prnull(fp2);
                prnull(fp2);
                prnull(fp2);
                prcr(fp2);
                prlf(fp2);
        }
        prcr(fp2);
        prlf(fp2);
        prff(fp2);
        close( fp2 );
}



char mask[8] = { 
        '\200','\100','\040','\020','\010','\04','\02','\01' } 
;





prconv( buf,size,cnt )
/* Denna rutin konverterar den area som buf pekar p} till n}ldata i     */
/* bufferten utbuf[]. Size {r bredden p} bilden i bytes och cnt {r      */
/* bredden i pixels.                                                    */
char *buf;
int size,cnt;
{
        int i,j,k,l,m,p,q,h;

        for( i=0; i<2048; i++ ){
                utbuf[i] = 0;           /* clear area */
        }

        p=0;
        h=0;
        for( i=0; i<cnt ; i++ ) {
                q=64;
                k = 0;   /* clear data   */
                for( j=0; j<7 ; j++ ) {
                        /* build needle data    */
                        l = buf[ h+(j*size) ];
                        m = mask[ p ];
                        if( ( l & m ) != 0 ) k += q;
                        q=q/2;
                }

                if( inv_flag == 0 )
                    utbuf[i] = ~k & 127 ; /* invert and mask,paper is white */
                else   utbuf[i] = k & 127 ; 
                p++;
                if( p==8 ) {
                        p=0;
                        h++;
                }
        }
}

cstr()  /* clear string str     */
{
int i;

        for( i=0; i<sizeof(str); i++){
                str[i] = 0;
        }
}

char space[10] = { 
        ' ',' ',' ',' ',' ',' ',' ',' ',' ',' ' };

prout(fp,buf,cnt)
/* Skriver ut ett antal ( cnt ) bytes ur bufferten buf[] till filen fp */
int fp;
char *buf[];
int cnt;
{
        char tmp[6];
        int i;

        tmp[0]= 27;     /* esc  */
        tmp[1]= 42;     /* *    */
        tmp[2]= 39;     /* Triple density=39   */
        tmp[3]= ( cnt & 0xff ); /* low part of cnt      */
        tmp[4]= ( ( cnt >> 8 ) & 0xff ) ;       /* high part of cnt */
        write( fp, &tmp[0], 5 );        /* set graphic mode     */
        write( fp, &buf[0], cnt );      /* write needle data    */

}

prcr(fp)
/* Skriver vagnretur till filen fp      */
int fp;
{
        char tmp[4];

        tmp[0]= 13;     /* cr  */
        tmp[1]= 0;
        write( fp, &tmp[0], 2 );        /* cr   */

}



prlf(fp)
/* Beordrar pappersmatning med 7 n}lars h|jd ( line-feed )      */
int fp;
{
        char tmp[6];

        tmp[0]= 27;     /* cr  */
        tmp[1]= 65;
        tmp[2]= 7;      /* mata 7 n}lar */
        tmp[3]= 10;     /* linefeed     */
        tmp[4]= 0;
        tmp[5]= 0;
        write( fp, &tmp[0], 6 );        /* lf   */

}


prnull(fp)
int fp;
{
        char tmp[6];

        tmp[0]= 0; 
        tmp[1]= 0;
        tmp[2]= 0;
        tmp[3]= 0;
        tmp[4]= 0;
        tmp[5]= 0;
        write( fp, &tmp[0], 6 );

}

prff(fp)
int fp;
{
        char tmp[4];

        tmp[0]= 12;     /* cr  */
        tmp[1]= 0;
        write( fp, &tmp[0], 2 );        /* cr   */

}
