/************************************************************************/
/*      wpp.c            window-print picture file			*/
/*									*/
/*	Programmet l{ser fil och konverterar f|r grafisk utskrift	*/
/*	p} printer.							*/
/*	Syntax: wpp filnamn device bredd grafikformat [invert]		*/
/*			grafikformat:	0 Epson normal punktgrafik	*/
/*					1 Epson t{t punktgrafik		*/
/*					5 Epson x:y 1:1 pg.(ej MX80/100)*/
/*				      100 Hewlett-Packard laserprinter	*/
/*									*/
/*        by Rolf Loh							*/
/*									*/
/************************************************************************/

#include <stdio.h>
#include <sgtty.h>
#include "fedef.h"                      /* Include definitions          */


/************************************************************************/
/*                         D A T A      A R E A                         */

char picarea[ 0x7fff ];         /* work fontfile area                   */
char utbuf[2048];
char inbuf[2048];
/*                                                                      */ 


#define NCC     8

typedef unsigned char   uchar_t;
#define IXOFF   0010000


#define TIOC    ('T'<<8)
#define TCGETA  (TIOC|1)
#define TCSETA  (TIOC|2)

/*
 * Ioctl control packet
 */
struct termio {
        unsigned short  c_iflag;        /* input modes */
        unsigned short  c_oflag;        /* output modes */
        unsigned short  c_cflag;        /* control modes */
        unsigned short  c_lflag;        /* line discipline modes */
        char            c_line;         /* line discipline */
        uchar_t         c_ccs[NCC];     /* control chars */
};



char c;
int bytcnt;
int pixwidth,i,x;
char str[64];
int inv_flag;

struct sgttyb ttysave;
int linje,paptr;
long filptr;
int bwidth;
int graphtype;


/************************************************************************/



main(argc, argv)
int argc;
char *argv[];
{
        struct termio term;
        int fp1,fp2;

        if( (argc < 5) | (argc > 6) ) {
            printf("Syntax: wpp filnamn device bredd grafiktyp [invert]\n");
            printf("grafiktyp:   0 Epson normal punktgrafik\n");
            printf("             1 Epson t{t punktgrafik\n");
	    printf("             5 Epson x:y 1:1 pg.(ej MX80/100)\n");
	    printf("           100 Hewlett-Packard laserprinter\n\n");
            exit(1);
        }


        /* avkoda kommandoraden         */
        if(( fp1 = open(argv[1],0)) == -1) {
                error("wpp: can't open %s\n", argv[1]);
                exit(0);
        }
        if(( fp2 = creat(argv[2], 0644)) ==-1) {
                error("wpp: can't create %s\n", argv[2]);
                exit(0);
        }

        if( sscanf( argv[3],"%d",&pixwidth ) != 1 ){
                error("wpp: wrong argument %s\n",argv[3]);
                exit(0);
        }
        bwidth = pixwidth/8;

        if( sscanf( argv[4],"%d",&graphtype ) != 1 ){
                error("wpp: wrong argument %s\n",argv[4]);
                exit(0);
        }

        /* kolla om bilden skall inverteras */
        inv_flag=0;
        if( argc == 6 ) inv_flag = 1;


 
        /* set tty-mode */

        ioctl(fp2,TIOCGETP,&ttysave);
        ttysave.sg_flags |= RAW;        /* s{tt raw mode        */
        ttysave.sg_flags &= ~ECHO;      /* ta bort echo         */
        ioctl(fp2,TIOCSETP,&ttysave);

        /* disable automatic XON/XOFF */
        /* should be done after setting the line to RAW mode */
        ioctl(fp2,TCGETA,&term);
        term.c_iflag &= ~IXOFF;
        ioctl(fp2,TCSETA,&term);
    

        if(graphtype<100) fontprint(fp1, fp2); 	/* Epson skriv ut bilden   */
	else HPprint(fp1,fp2,argv[4]);		/* HP laserskrivare	*/

        ttysave.sg_flags &= ~RAW;       /* }terst{ll porten     */
        ttysave.sg_flags |= ECHO;
        ioctl(fp2,TIOCSETP,&ttysave);

        cstr();
	if(graphtype<100){
        /* ]terst{ll Epson printer (ESC @)  */
	        sprintf( &str[0], "%c%c",27,64);
        	write(fp2, &str[0], 2 );
	}
        close( fp2 );

        exit(0);
}

/****************************************************************/
error(s1,s2)    /* skriver ut felutskrift.s1=str{ng,s2s{tts in i */
char *s1,*s2;   /* s1 med %s funktonen i printf                 */
{
        printf(s1, s2);
        printf("\n");
        exit(1);
}


fontprint(fp1,fp2)
/* Epson printers:							*/
/* Denna rutin l{ser det antal bytes					*/
/* som kr{vs f|r att skriva med 7 n}lar |ver bildens bredd.D{refter     */
/* konverteras dessa bytes f|r att skrivas ut.                          */
int fp1,fp2;
{



        filptr = lseek(fp1,0L,2); /* position at end of file      */
        bytcnt = 0xfff;
        while( bytcnt > 0 ) {
                bytcnt = readin(fp1);            /* Read data            */
                prconv( &picarea[0], bwidth,pixwidth);
                if( bytcnt <= 0 ) break;
                prout( fp2,&utbuf[0], pixwidth );
                prnull(fp2);
                prnull(fp2);
                prnull(fp2);
                prcr(fp2);
                prlf(fp2);
        }
        prff(fp2);
}




readin(fp1)
/* l{ser in data f|r 7 n}lar fr}n filen fp1.Datat l{sesfr}n slutet av filen */
/* ock fram}t i block som motsvarar bildens bredd.Returnerar pos   i filen  */
int fp1;
{
        paptr = 0;
        for(linje=0; linje<7; linje++){
             filptr -= bwidth;
             lseek(fp1, filptr,0);      /* pos back     */
             bytcnt = read( fp1,&picarea[paptr],bwidth); /*Read data*/
             paptr += bytcnt;
        }
        return(filptr);
}







char mask[8] = { 
        '\200','\100','\040','\020','\010','\04','\02','\01' } 
;

prconv( buf,size,cnt )
/* Denna rutin konverterar den area som buf pekar p} till n}ldata i     */
/* bufferten utbuf[]. Size {r bredden p} bilden i bytes och cnt {r      */
/* bredden i pixels.                                                    */
char *buf;
int size,cnt;
{
        int i,j,k,l,m,p,q,h;

        for( i=0; i<2048; i++ ){
                utbuf[i] = 0;           /* clear area */
        }

        p=0;
        h=0;
        for( i=0; i<cnt ; i++ ) {
                q=64;
                k = 0;   /* clear data   */
                for( j=0; j<7 ; j++ ) {
                        /* build needle data    */
                        l = buf[ h+(j*size) ];
                        m = mask[ p ];
                        if( ( l & m ) != 0 ) k += q;
                        q=q/2;
                }

                if( inv_flag == 0 )
                    utbuf[i] = ~k & 127 ; /* invert and mask,paper is white */
                else   utbuf[i] = k & 127 ; 
                p++;
                if( p==8 ) {
                        p=0;
                        h++;
                }
        }
}

cstr()  /* clear string str     */
{
int i;

        for( i=0; i<sizeof(str); i++){
                str[i] = 0;
        }
}

char space[10] = { 
        ' ',' ',' ',' ',' ',' ',' ',' ',' ',' ' };

prout(fp,buf,cnt)
/* Skriver ut ett antal ( cnt ) bytes ur bufferten buf[] till filen fp */
int fp;
char *buf[];
int cnt;
{
	char tmp[6];

        int i;

        if( graphtype < 2 ) {
                tmp[0]= 0;
                tmp[1]= 27;     /* ESC  */
                if( graphtype == 0 ) tmp[2] = 75; /* normal punktgrafik */
                else tmp[2] = 76;       /* f|rminskad punktgrafik */
        }
        else {
                tmp[0]= 27;     /* esc  */
                tmp[1]= 42;     /* *    */
                tmp[2]= graphtype;      /* set graphic command  */
        }
        tmp[3]= ( cnt & 0xff ); /* low part of cnt      */
        tmp[4]= ( ( cnt >> 8 ) & 0xff ) ;       /* high part of cnt */
	if( i = read( fp, &inbuf[0],256 ) > 0 ) {
		printf("wpp: printer sends %x\n",&inbuf[0]);
		sleep(4);
	}
        write( fp, &tmp[0], 5 );        /* set graphic mode     */
        write( fp, &buf[0], cnt );      /* write needle data    */

}




HPprint(fp1,fp2,argv)
int fp1,fp2;
char *argv;
{
	char tmp[10];
	int i;

	/* Raster graphics resolution	*/
	tmp[0] = 27;    /* ESC  */
        tmp[1] = 42;    /* *    */
        tmp[2] = 't';   /* t    */
        tmp[3] = 0;
	strncat(argv,&tmp[0],2);
        tmp[5] = 'W';

        /* Start Raster graphics        */
        tmp[0] = 27;    /* ESC  */
        tmp[1] = 42;    /* *    */
        tmp[2] = 'r';   /* r    */
        tmp[3] = '0';
	tmp[4] = '0';
        tmp[5] = 'A';

        filptr = lseek(fp1,0L,2); /* position at end of file      */
        bytcnt = 0xfff;
        while( bytcnt > 0 ) {
		filptr -= bwidth;
		lseek(fp1,filptr,0);	/* pos. back */
		bytcnt = read(fp1,&picarea[0],bwidth);
		tmp[0] = 27;	/* ESC	*/
		tmp[1] = 42;	/* *	*/
		tmp[2] = 'b';	/* b	*/
		tmp[3] = (bytcnt & 0xff );	/* low part of byte-count */
		tmp[4] = ((bytcnt >> 8) & 0xff);/* high part	*/
		tmp[5] = 'W';
	write(fp2,&tmp[0],6);
	if(inv_flag){
		for(i=0;i<bytcnt;i++)
			picarea[i] = ~(picarea[i]);	/* invert */
	}
	write(fp2,&picarea[0],bytcnt);
        }

        prff(fp2);		/* mata papper	*/
}


/******************************************************************/
prcr(fp)
/* Skriver vagnretur till filen fp      */
int fp;
{
        char tmp[4];

        tmp[0]= 13;     /* cr  */
        tmp[1]= 0;
        write( fp, &tmp[0], 2 );        /* cr   */

}



prlf(fp)
/* Beordrar pappersmatning med 7 n}lars h|jd ( line-feed )      */
int fp;
{
        char tmp[6];

        tmp[0]= 27;     /* cr  */
        tmp[1]= 65;
        tmp[2]= 7;      /* mata 7 n}lar */
        tmp[3]= 10;     /* linefeed     */
        tmp[4]= 0;
        tmp[5]= 0;
        write( fp, &tmp[0], 6 );        /* lf   */

}


prnull(fp)
int fp;
{
        char tmp[6];

        tmp[0]= 0; 
        tmp[1]= 0;
        tmp[2]= 0;
        tmp[3]= 0;
        tmp[4]= 0;
        tmp[5]= 0;
        write( fp, &tmp[0], 6 );

}

prff(fp)
int fp;
{
        char tmp[4];

        tmp[0]= 12;     /* cr  */
        tmp[1]= 0;
        write( fp, &tmp[0], 2 );        /* cr   */

}

